/*
    HBRC Challenge Phase II program for the Neato XV-11.
    Boris Kudryavtsev
    boris@buzz-news.com
*/

#include "ros/ros.h"
#include "sensor_msgs/Range.h"
#include "sensor_msgs/LaserScan.h"
#include "geometry_msgs/Twist.h"

float drop_l = 0.0;
float drop_r = 0.0;

double angle = 30.0;

const float DROP_THRESHOLD = 0.075;
const float DRIVE_SPEED = 0.095;

ros::Publisher pubCmd_vel;

enum STATE { START, STOP };
STATE state = START;

static bool dropFound = false;


void setVel ( float vel )
{
  //ROS_INFO("Deb %f", vel);	
  geometry_msgs::Twist msg;
  msg.linear.x = vel;
  pubCmd_vel.publish ( msg );
}
void turn ( float rad_sec )
{
  geometry_msgs::Twist msg_twist;
  msg_twist.angular.z = rad_sec;
  pubCmd_vel.publish ( msg_twist );
}

void drop_r_Callback(const sensor_msgs::Range::ConstPtr& msg)
{
  ROS_INFO("I heard right: [%f]", msg->range);
  drop_r = msg->range;
  if (drop_r > DROP_THRESHOLD) 
  {
      dropFound = true;
      setVel ( 0.0 );
  }
}

void drop_l_Callback(const sensor_msgs::Range::ConstPtr& msg)
{
  ROS_INFO("I heard left: [%f]", msg->range);
  drop_l = msg->range;
  if (drop_l > DROP_THRESHOLD) 
  {
      dropFound = true;
      setVel ( 0.0 );
  }
}
void laser_scan(const sensor_msgs::LaserScan::ConstPtr& msg)
{
	int min = -1;
	float minr = 1000.0f;
	for (unsigned int i = 0; i <  msg->ranges.size(); i++) {
		//ROS_INFO("Laser Scan: %u %f", i, msg->ranges[i]);
		if (msg->ranges[i] > 0.01 && msg->ranges[i] < minr) {
			minr = msg->ranges[i];
			min = i;		
		}
	}
	double rangle = min * msg->angle_increment + msg->angle_min;
	while (rangle > +M_PI) { rangle -= 2 * M_PI; }
	while (rangle < -M_PI) { rangle += 2 * M_PI; }
	rangle *= 180.0/3.14159;
	angle = rangle;
	ROS_INFO("Angle: %f %f %d", rangle, minr, min);
}

int main(int argc, char **argv)
{
  ros::init(argc, argv, "phase2");

  ros::NodeHandle n;

  pubCmd_vel = n.advertise<geometry_msgs::Twist>("cmd_vel", 1);

  ros::Subscriber subDrop_l = n.subscribe("drop_l", 1, drop_l_Callback);
  ros::Subscriber subDrop_r = n.subscribe("drop_r", 1, drop_r_Callback);
  ros::Subscriber baseScan = n.subscribe("base_scan", 1, laser_scan);

  ros::Rate r(25);
  bool die = false;
  while(ros::ok())
  {
	r.sleep();
	ros::spinOnce();

	switch ( state ) 
	{ 
   		case START:
		   while (1) {
			    ros::spinOnce();
				ros::Duration(0.1).sleep();
				ROS_INFO("My angle %f", angle);
				if (angle < -15.0)
				{
				  turn(-0.5);
				  if ( dropFound ) 
				  {
					state = STOP;
					break;
			          }
				} 
				else if (angle > 15.0)
				{
				  turn(0.5);
  				  if ( dropFound ) 
				  {
					state = STOP;
					break;
			          }
				}
				else if(angle > -15.0 && angle < 15.0)
				{
				  setVel ( DRIVE_SPEED );
				  if ( dropFound ) 
				  {
					state = STOP;
					break;
			          }
				}
			     } 
			state = STOP;
			break;
		case STOP:
			setVel ( -0.5 );
			ROS_INFO ("Reverse");
			ros::Duration(0.55).sleep();
			setVel ( 0.0 );	
			die = true;
			break;
		
        }
	if ( die )
	{	
		ROS_INFO ( "Exit" ); 
		break;
	}
  }
  while(ros::ok()) {   
    ros::Duration(0.1).sleep();
    ros::spinOnce();
    setVel ( 0.0 );	
  }
}
